//==========================================================================
// File : MSOEApi.pas
//
// Definitions for Windows Mail API in Delphi, according to the Windows 2003
// version of the Microsoft Platform SDK
//
// Author : K. Giddings
//
//
// This source code is the property of its authors and may not be
// redistributed or reproduced in any form without the written
// permission of the authors.
//
// You may freely use this unit in the creation of any software. 
//
// Copyright (C) 2007 GP Software (Thame) Ltd
//
// This software is provided 'as-is', without any express or implied
// warranty.  In no event will the authors be held liable for any damages
// arising from the use of this software.
//
//==========================================================================
unit MSOEApi;

interface

uses Windows, Messages, ActiveX, MimeOle;

const
  //+-------------------------------------------------------------------------
  // GUID Definitions
  //--------------------------------------------------------------------------
  CLSID_OutlookExpress : TGUID = '{3338DF69-4660-11d1-8A8D-00C04FB951F3}';
  CLSID_Envelope       : TGUID = '{0006F01A-0000-0000-C000-000000000046}';
  IID_IOutlookExpress  : TGUID = '{3338DF6A-4660-11d1-8A8D-00C04FB951F3}';
  CLSID_StoreNamespace : TGUID = '{E70C92A9-4BFD-11d1-8A95-00C04FB951F3}';
  IID_IStoreNamespace  : TGUID = '{E70C92AA-4BFD-11d1-8A95-00C04FB951F3}';
  IID_IStoreFolder     : TGUID = '{E70C92AC-4BFD-11d1-8A95-00C04FB951F3}';

  SID_OutlookExpress = '{3338DF69-4660-11d1-8A8D-00C04FB951F3}';
  SID_Envelope = '{0006F01A-0000-0000-C000-0000000046}';
  SID_IOutlookExpress = '{3338DF6A-4660-11d1-8A8D-00C04FB951F3}';
  SID_StoreNamespace = '{E70C92A9-4BFD-11d1-8A95-00C04FB951F3}';
  SID_IStoreNamespace = '{E70C92AA-4BFD-11d1-8A95-00C04FB951F3}';
  SID_IStoreFolder = '{E70C92AC-4BFD-11d1-8A95-00C04FB951F3}';
  SID_IStoreNamespace2 = '{DECAED7F-2DA8-4074-B62A-DB6D8C82C385}';
  SID_IStoreFolder2 = '{5642F741-46B7-4A22-B496-F064E4C5F302}';

  //+-------------------------------------------------------------------------
  // Errors Definition Macros
  //--------------------------------------------------------------------------
  FACILITY_INTERNET = 12;

const
  //+-------------------------------------------------------------------------
  // MSOEAPI Failure Return Values
  //--------------------------------------------------------------------------
  MSOEAPI_E_FILE_NOT_FOUND = (SEVERITY_ERROR shl 31) or (FACILITY_INTERNET shl 16) or $CF65;
  MSOEAPI_E_STORE_INITIALIZE  = (SEVERITY_ERROR shl 31) or (FACILITY_INTERNET shl 16) or $CF66;
  MSOEAPI_E_INVALID_STRUCT_SIZE  = (SEVERITY_ERROR shl 31) or (FACILITY_INTERNET shl 16) or $CF67;
  MSOEAPI_E_CANT_LOAD_MSOERT  = (SEVERITY_ERROR shl 31) or (FACILITY_INTERNET shl 16) or $CF68;
  MSOEAPI_E_CANT_LOAD_INETCOMM  = (SEVERITY_ERROR shl 31) or (FACILITY_INTERNET shl 16) or $CF69;
  MSOEAPI_E_CANT_LOAD_MSOEACCT  = (SEVERITY_ERROR shl 31) or (FACILITY_INTERNET shl 16) or $CF70;
  MSOEAPI_E_CANT_MSOERT_BADVER  = (SEVERITY_ERROR shl 31) or (FACILITY_INTERNET shl 16) or $CF71;
  MSOEAPI_E_CANT_INETCOMM_BADVER  = (SEVERITY_ERROR shl 31) or (FACILITY_INTERNET shl 16) or $CF72;
  MSOEAPI_E_CANT_MSOEACCT_BADVER  = (SEVERITY_ERROR shl 31) or (FACILITY_INTERNET shl 16) or $CF73;

  //+-------------------------------------------------------------------------
  // Strings
  //--------------------------------------------------------------------------
  STR_MSOEAPI_INSTANCECLASS = 'OutlookExpressHiddenWindow';
  STR_MSOEAPI_IPSERVERCLASS = 'OutlookExpressInProccessServer';
  STR_MSOEAPI_INSTANCEMUTEX = 'Local\\OutlookExpress_InstanceMutex_101897';
  STR_MSOEAPI_DLLNAME       = 'MSOE.DLL';
  STR_MSOEAPI_START         = 'CoStartOutlookExpress';
  STR_MSOEAPI_SHUTDOWN      = 'CoShutdownOutlookExpress';
  STR_MSOEAPI_CREATE        = 'CoCreateOutlookExpress';

  //+-------------------------------------------------------------------------
  // Function Typedefs
  //--------------------------------------------------------------------------
type
  TfnStart = function (dwFlags : DWORD; pszCmdLine : LPCSTR; nCmdShow : integer) : HResult; stdcall;
  TfnShutdown = function (dwReserved : DWORD) : HResult; stdcall;
  TfnCreate = function(pUnkOuter : IUnknown; var ppUnknown : IUnknown) : HResult; stdcall;


//+-------------------------------------------------------------------------
// Constants
//--------------------------------------------------------------------------
const
  MSOEAPI_ACDM_CMDLINE    : DWORD = 1;
  MSOEAPI_ACDM_NOTIFY     : DWORD = 2;
  MSOEAPI_ACDM_ODBNOTIFY  : DWORD = 3;
  MSOEAPI_ACDM_STGNOTIFY  : DWORD = 4;

type
  HENUMSTORE = THandle;
  LPHENUMSTORE = ^HENUMSTORE;

const
  //+---------------------------------------------------------------------
  // Flags used in IOutlookExpress::Start
  //----------------------------------------------------------------------
  MSOEAPI_START_SHOWSPLASH         =  $00000001;
  MSOEAPI_START_MESSAGEPUMP        =  $00000002;
  MSOEAPI_START_ALLOWCOMPACTION    =  $00000004;
  MSOEAPI_START_INSTANCEMUTEX      =  $00000008;
  MSOEAPI_START_SHOWERRORS         =  $00000010;
  MSOEAPI_START_APPWINDOW          =  $00000020;
  MSOEAPI_START_DEFAULTIDENTITY    =  $00000040;
  MSOEAPI_START_ALREADY_RUNNING    =  $00000080;
  MSOEAPI_START_STOREVALIDNODELETE =  $00000100;
  MSOEAPI_START_NOSTORE            =  $00000200;

type
  MSOEAPI_START_APPLICATION  = DWORD;
  MSOEAPI_START_COMOBJECT    = DWORD;

type
  IOutlookExpress = interface(IUnknown)
    [SID_OutlookExpress]
    function Start(dwFlags : DWORD; pszCmdLine : LPCSTR; nCmdShow : integer) : HResult; stdcall;
  end;

  STOREFOLDERID = DWORD;
  LPSTOREFOLDERID = ^STOREFOLDERID;

const
  //+---------------------------------------------------------------------
  // Some Special FOLDER Values
  //----------------------------------------------------------------------
  FOLDERID_ROOT     = 0;
  FOLDERID_INVALID  = $ffffffff;

  //+---------------------------------------------------------------------
  // Some Special MESSAGEID values
  //----------------------------------------------------------------------
  MESSAGEID_INVALID : DWORD = $ffffffff;
  MESSAGEID_FIRST   : DWORD = $ffffffff;

  //+---------------------------------------------------------------------
  // Consts
  //----------------------------------------------------------------------
  CCHMAX_FOLDER_NAME = 256;

  //+---------------------------------------------------------------------
  // Notification Messages
  //----------------------------------------------------------------------
  WM_FOLDERNOTIFY    = WM_USER + 1600; // IStoreNamespace Notification - lparam=LPFOLDERNOTIFYEX, client must call CoTaskMemFree(lParam)")
  WM_NEWMSGS         = WM_USER + 1650; // IStoreFolder Notification: wParam=MESSAGEID, lParam=Reserved")
  WM_DELETEMSGS      = WM_USER + 1651; // IStoreFolder Notification: wParam=prgdwMsgId, lParam=cMsgs")
  WM_DELETEFOLDER    = WM_USER + 1652; // IStoreFolder Notification: wParam=STOREFOLDERID or HFOLDER")

  // If wParam is equal to 0xffffffff then that means that all messages in the folder
  // were either marked as read (WM_MARKEDASREAD) or unread (WM_MARKEDASUNREAD)
  WM_MARKEDASREAD    = WM_USER + 1653; // IStoreFolder Notification: wParamprgdwMsgId, lParam=cMsgs")
  WM_MARKEDASUNREAD  = WM_USER + 1654; // IStoreFolder Notification: wParamprgdwMsgId, lParam=cMsgs")

  //+---------------------------------------------------------------------
  // FOLDERNOTIFYTYPE
  //----------------------------------------------------------------------
type
  FOLDERNOTIFYTYPE = DWORD;

const
  NEW_FOLDER = 1;
  DELETE_FOLDER = 2;
  RENAME_FOLDER = 3;
  MOVE_FOLDER = 4;
  UNREAD_CHANGE = 5;
  IMAPFLAG_CHANGE = 6;
  UPDATEFLAG_CHANGE = 7;
  FOLDER_PROPS_CHANGED = 8;

type
  //+---------------------------------------------------------------------
  // MESSAGEID
  //----------------------------------------------------------------------
  MESSAGEID = DWORD;
  LPMESSAGEID = ^MESSAGEID;

  //+---------------------------------------------------------------------
  // FOLDERNOTIFYEX
  //----------------------------------------------------------------------
  FOLDERNOTIFYEX = record
    ftype : FOLDERNOTIFYTYPE;
    idFolderOld : STOREFOLDERID;                // not used for CREATEFOLDER and DELETEFOLDER
    idFolderNew : STOREFOLDERID;
  end;
  LPFOLDERNOTIFYEX = ^FOLDERNOTIFYEX;

  //+---------------------------------------------------------------------
  // MESSAGEIDLIST - A list of message ids (DWORDs)
  //----------------------------------------------------------------------
  MESSAGEIDLIST = record
    cbSize : DWORD;                 // Sizeof this structure
    cMsgs : DWORD;                  // Number of elements in prghMessage
    prgdwMsgId : LPMESSAGEID; // Array of handles to messages
  end;
  LPMESSAGEIDLIST = ^MESSAGEIDLIST;

  //+---------------------------------------------------------------------
  // SPECIALFOLDER - How to identify a special folder
  //----------------------------------------------------------------------
  SPECIALFOLDER = DWORD;

const
  FOLDER_NOTSPECIAL = -1;                     // Not a special folder
  FOLDER_INBOX = 0;                           // Default receive folder
  FOLDER_OUTBOX = 1;                          // Default send folder
  FOLDER_SENT = 2;                            // Things that have been sent
  FOLDER_DELETED = 3;                         // Things that have been deleted
  FOLDER_DRAFT = 4;                           // Things that are in progress
  FOLDER_MAX = 5;                             // Don't use

type
  FOLDERPROPS = record
    cbSize : DWORD;                     // Size of this structure
    dwFolderId : STOREFOLDERID;         // Handle of this folder
    cSubFolders : integer;              // Number of sub-folders
    sfType :  SPECIALFOLDER;            // Special folder type
    cUnread : DWORD;                    // Number of un-read messages
    cMessage : DWORD;                   // Total # of messages
    szName : array [0..CCHMAX_FOLDER_NAME - 1] of char; // The Folder Name
  end;

  LPFOLDERPROPS = ^FOLDERPROPS;

const
  NAMESPACE_INITIALIZE_CURRENTIDENTITY = $00000001;

  // CLocalStore::CopyMoveMessages flags - dwFlags == 0 is copy
  CMF_MOVE                    = $0001; // msgs deleted from src fldr after copy
  CMF_DELETE                  = $0002; // same as CMF_MOVE but uses delete string for status

  //+---------------------------------------------------------------------
  // Message State Flags used in MESSAGEINFO::dwState
  //----------------------------------------------------------------------
  MSG_DELETED                  = $0001;  // The message has been deleted, compaction will remove it
  MSG_UNREAD                   = $0002;  // The message is marked as un-read
  MSG_SUBMITTED                = $0004;  // For messages waiting to be sent: OUTBOX ONLY
  MSG_UNSENT                   = $0008;  // For msgs-in-progress saved to a folder
  MSG_RECEIVED                 = $0010;  // For messages that came from a server
  MSG_NEWSMSG                  = $0020;  // For news messages
  MSG_NOSECUI                  = $0040;  // For messages where the user wants to die
  MSG_VOICEMAIL                = $0080;  // The message has the X-Voicemail header set...
  MSG_REPLIED                  = $0100;  // The message has been replied to
  MSG_FORWARDED                = $0200;  // The message has been forwarded to
  MSG_RCPTSENT                 = $0400;  // S/MIME Receipt has been sent
  MSG_FLAGGED                  = $0800;  // The message is currently flaged
  MSG_LAST                     = $0200;  // ** Keep this updated!! **
  MSG_EXTERNAL_FLAGS           = $00fe;
  MSG_FLAGS                    = $000f;

  COMMITSTREAM_REVERT          = $00000001; // Don't add this stream/message to the folder
  MSGPROPS_FAST                = $00000001;

type
  IStoreFolder = interface;

  IProgressNotify = interface(IUnknown)
    ['{a9d758a0-4617-11cf-95fc-00aa00680db4}']
    function OnProgress(dwProgressCurrent : DWORD; dwProgressMaximum : DWORD; fAccurate : BOOL; fOwner : BOOL) : HResult; stdcall;
  end;

  IStoreNamespace = interface(IUnknown)
    [SID_IStoreNamespace]
    function Initialize(hwndOwner : HWND; dwReserved : DWORD) : HResult; stdcall;
    function GetDirectory(pszPath : LPSTR; cchMaxPath : DWORD) : HREsult; stdcall;
    function OpenSpecialFolder(sfType : SPECIALFOLDER; dwReserved : DWORD; out ppFolder : IStoreFolder) : HResult; stdcall;
    function OpenFolder(dwFolderId : STOREFOLDERID; dwReserved : DWORD; out ppFolder : IStoreFolder) : HResult; stdcall;
    function CreateFolder(dwParentId : STOREFOLDERID;pszName : LPCSTR; dwReserved : DWORD;out pdwFolderId : STOREFOLDERID) : HResult; stdcall;
    function RenameFolder(dwFolderId : STOREFOLDERID; dwReserved : DWORD; pszNewName : LPCSTR) : HResult; stdcall;
    function MoveFolder(dwFolderId : STOREFOLDERID; dwParentId : STOREFOLDERID; dwReserved : DWORD) : HResult; stdcall;
    function DeleteFolder(dwFolderId : STOREFOLDERID; dwReserved : DWORD) : HResult; stdcall;
    function GetFolderProps(dwFolderId : STOREFOLDERID; dwReserved : DWORD; var pProps : FOLDERPROPS) : HResult; stdcall;
    function CopyMoveMessages(pSource : IStoreFolder; pDest : IStoreFolder; pMsgIdList : LPMESSAGEIDLIST; dwFlags : DWORD; dwFlagsRemove : DWORD; pProgress : IProgressNotify) : HResult; stdcall;
    function RegisterNotification(dwReserved : DWORD; wnd : HWND) : HResult; stdcall;
    function UnregisterNotification(dwReserved : DWORD; wnd : HWND) : HResult; stdcall;
    function CompactAll(dwReserved : DWORD) : HResult; stdcall;
    function GetFirstSubFolder(dwFolderId : STOREFOLDERID; var pProps: FOLDERPROPS; out phEnum : HENUMSTORE) : HResult; stdcall;
    function GetNextSubFolder(hEnum : HENUMSTORE; var pProps : FOLDERPROPS) : HResult; stdcall;
    function GetSubFolderClose(hEnum : HENUMSTORE) : HResult; stdcall;
  end;

  //----------------------------------------------------------------------
  // MESSAGEPROPS
  //----------------------------------------------------------------------
  MESSAGEPROPSSIZE = DWORD;
  MESSAGEPROPS = record
    cbSize : MESSAGEPROPSSIZE;                    // Size of this structure
    dwReserved : DWORD;                 // Reserved don't use
    dwMessageId : MESSAGEID;                // Handle to this message
    dwLanguage : DWORD;                 // Codepage of this message
    dwState : DWORD;                    // MSG_xxx Flags

    // Things you don't get when MSGPROPS_FAST
    cbMessage : DWORD;                  // Size of the message
    priority : IMSGPRIORITY;                   // Message Priority
    ftReceived : FILETIME;                 // When the message was received
    ftSent: FILETIME;                     // When the message was sent
    pszSubject : LPSTR;                 // The subject of the message
    pszDisplayTo : LPSTR;               // The display to line of the message
    pszDisplayFrom : LPSTR;             // The display from line of the message
    pszNormalSubject : LPSTR;           // The normalized subject of the message
    dwFlags : DWORD;                    // IMF_xxx Flags defined in mimeole.idl
    pStmOffsetTable : IStream;            // Use with IMimeMessage::LoadOffsetTable
  end;
  LPMESSAGEPROPS = ^MESSAGEPROPS;

  //+---------------------------------------------------------------------
  // HBATCHLOCK
  //----------------------------------------------------------------------
  HBATCHLOCK = THandle;
  LPHBATCHLOCK = ^HBATCHLOCK;

  IStoreFolder = interface(IUnknown)
    [SID_IStoreFolder]
    function GetFolderProps(dwReserved : DWORD; var pProps : FOLDERPROPS) : HResult;stdcall;
    function GetMessageProps(dwMessageId : MESSAGEID; dwFlags : DWORD; var pProps : MESSAGEPROPS) : HResult;stdcall;
    function FreeMessageProps(var pProps : MESSAGEPROPS) : HResult;stdcall;
    function DeleteMessages(pMsgIdList : LPMESSAGEIDLIST; dwReserved : DWORD; pProgress : IProgressNotify) : HResult;stdcall;
    function SetLanguage(dwLanguage : DWORD; dwReserved : DWORD; pMsgIdList : LPMESSAGEIDLIST) : HResult;stdcall;
    function MarkMessagesAsRead(fRead : BOOL; dwReserved : DWORD; pMsgIdList : LPMESSAGEIDLIST) : HResult;stdcall;
    function SetFlags(pMsgIdList : LPMESSAGEIDLIST; dwState : DWORD; dwStatemask : DWORD; var prgdwNewFlags : DWORD) : HResult;stdcall;
    function OpenMessage(dwMessageId : MESSAGEID; const riid : TIID; out ppvObject) : HResult;stdcall;
    function SaveMessage(const riid : TIID; out pvObject; dwMsgFlags : DWORD; out pdwMessageId : MESSAGEID) : HResult;stdcall;
    function BatchLock(dwReserved : DWORD; out phBatchLock : HBATCHLOCK) : HResult;stdcall;
    function BatchFlush(dwReserved : DWORD; BatchLock : HBATCHLOCK) : HResult;stdcall;
    function BatchUnlock(dwReserved : DWORD; BatchLock : HBATCHLOCK) : HResult;stdcall;
    function CreateStream(BatchLock : HBATCHLOCK; dwReserved : DWORD; out ppStream : IStream; out pdwMessageId : MESSAGEID) : HResult;stdcall;
    function CommitStream(BatchLock : HBATCHLOCK; dwFlags : DWORD; dwMsgFlags : DWORD; pStream : IStream; dwMessageId : MESSAGEID; pMessage : IMimeMessage) : HResult;stdcall;
    function RegisterNotification(dwReserved : DWORD; wnd : HWND) : HResult;stdcall;
    function UnregisterNotification(dwReserved : DWORD; wnd : HWND) : HResult;stdcall;
    function Compact(dwReserved : DWORD) : HResult;stdcall;
    function GetFirstMessage(dwFlags : DWORD; dwMsgFlags : DWORD; dwMsgIdFirst : MESSAGEID; var pProps : MESSAGEPROPS; out phEnum : HENUMSTORE) : HResult;stdcall;
    function GetNextMessage(hEnum : HENUMSTORE; dwFlags : DWORD; var pProps : MESSAGEPROPS) : HResult;stdcall;
    function GetMessageClose(hEnum : HENUMSTORE) : HResult;stdcall;
  end;

  IStoreNamespace2 = interface(IStoreNamespace)
    function GetMessageID(pszPath : LPCWSTR; out pdwFolderId : STOREFOLDERID; out pdwMessageId : MESSAGEID) : HResult;stdcall;
    function GetParentId(dwFolderId : STOREFOLDERID; out pdwParentId : STOREFOLDERID) : HResult;stdcall;
    function GetDirectoryW(pwszPath : LPWSTR; cchMaxPath : DWORD) : HResult;stdcall;
    function CreateFolderW(dwParentId : STOREFOLDERID; pwszName : LPCWSTR; dwReserved : DWORD; out pdwFolderId : STOREFOLDERID) : HResult;stdcall;
    function RenameFolderW(dwFolderId : STOREFOLDERID; dwReserved : DWORD; pwszNewName : LPCWSTR) : HResult; stdcall;
  end;

  IStoreFolder2 = interface(IStoreFolder)
    [SID_IStoreFolder2]
    function GetMessagePath(dwMessageId : MESSAGEID; pszPath : LPWSTR; cchBufferSize : DWORD) : HResult; stdcall;
  end;

//+-------------------------------------------------------------------------")
// Outlook Express Exported C API Functions")
//--------------------------------------------------------------------------")
function CoStartOutlookExpress(dwFlags : DWORD; pszCmdLine : LPCSTR; nCmdShow : integer) : HResult;
function CoCreateOutlookExpress(pUnkOuter : IUnknown; out ppUnknown : IUnknown) : HResult; stdcall;

implementation

const
  msoe = 'msoe.dll';

function CoStartOutlookExpress(dwFlags : DWORD; pszCmdLine : LPCSTR; nCmdShow : integer) : HResult;
var
  _CoStartOutlookExpress : function(dwFlags : DWORD; pszCmdLine : LPCSTR; nCmdShow : integer) : HResult; stdcall;
  Dll : THandle;
begin
  Result := E_FAIL;
	Dll := LoadLibrary(msoe);
	if (Dll <> 0) then
  begin
		_CoStartOutlookExpress := GetProcAddress(Dll, 'CoStartOutlookExpress');
		if Assigned(_CoStartOutlookExpress) then
			Result := _CoStartOutlookExpress(dwFlags, pszCmdLine, nCmdShow);
		FreeLibrary(Dll);
  end;
end;

function CoCreateOutlookExpress(pUnkOuter : IUnknown; out ppUnknown : IUnknown) : HResult; stdcall;
var
  _CoCreateOutlookExpress : function(pUnkOuter : IUnknown; out ppUnknown : IUnknown) : HResult; stdcall;
  Dll : THandle;
begin
  Result := E_FAIL;
	Dll := LoadLibrary(msoe);
	if (Dll <> 0) then
  begin
		_CoCreateOutlookExpress := GetProcAddress(Dll, 'CoCreateOutlookExpress');
		if Assigned(_CoCreateOutlookExpress) then
			Result := _CoCreateOutlookExpress(pUnkOuter, ppUnknown);
		FreeLibrary(Dll);
  end;
end;

end.
